# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import absolute_import

import re  # noqa: F401

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from datetime import date

from pydantic import Field, StrictStr, conint, conlist

from typing import Optional

from synctera_client_ng.models.patch_payment_schedule import PatchPaymentSchedule
from synctera_client_ng.models.payment_list import PaymentList
from synctera_client_ng.models.payment_schedule import PaymentSchedule
from synctera_client_ng.models.payment_schedule_list import PaymentScheduleList

from synctera_client_ng.api_client import ApiClient
from synctera_client_ng.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class PaymentSchedulesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def create_payment_schedule(self, payment_schedule : Annotated[PaymentSchedule, Field(..., description="payment schedule to create")], **kwargs) -> PaymentSchedule:  # noqa: E501
        """Create a payment schedule  # noqa: E501

        Create a payment schedule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_payment_schedule(payment_schedule, async_req=True)
        >>> result = thread.get()

        :param payment_schedule: payment schedule to create (required)
        :type payment_schedule: PaymentSchedule
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaymentSchedule
        """
        kwargs['_return_http_data_only'] = True
        return self.create_payment_schedule_with_http_info(payment_schedule, **kwargs)  # noqa: E501

    @validate_arguments
    def create_payment_schedule_with_http_info(self, payment_schedule : Annotated[PaymentSchedule, Field(..., description="payment schedule to create")], **kwargs):  # noqa: E501
        """Create a payment schedule  # noqa: E501

        Create a payment schedule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_payment_schedule_with_http_info(payment_schedule, async_req=True)
        >>> result = thread.get()

        :param payment_schedule: payment schedule to create (required)
        :type payment_schedule: PaymentSchedule
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaymentSchedule, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'payment_schedule'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_payment_schedule" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['payment_schedule']:
            _body_params = _params['payment_schedule']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '201': "PaymentSchedule",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '409': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/payment_schedules', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_payment_schedules(self, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, id : Annotated[Optional[conlist(StrictStr)], Field(description="IDs. Multiple IDs can be provided as a comma-separated list.")] = None, description : Annotated[Optional[StrictStr], Field(description="Payment schedule description. String can be provided with wildcard.")] = None, account_id : Annotated[Optional[conlist(StrictStr)], Field(description="Originating account IDs. Multiple IDs can be provided as a comma-separated list.")] = None, customer_id : Annotated[Optional[conlist(StrictStr)], Field(description="The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.")] = None, **kwargs) -> PaymentScheduleList:  # noqa: E501
        """List payment schedules  # noqa: E501

        Get paginated list of payment schedules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_payment_schedules(limit, page_token, id, description, account_id, customer_id, async_req=True)
        >>> result = thread.get()

        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param id: IDs. Multiple IDs can be provided as a comma-separated list.
        :type id: List[str]
        :param description: Payment schedule description. String can be provided with wildcard.
        :type description: str
        :param account_id: Originating account IDs. Multiple IDs can be provided as a comma-separated list.
        :type account_id: List[str]
        :param customer_id: The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.
        :type customer_id: List[str]
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaymentScheduleList
        """
        kwargs['_return_http_data_only'] = True
        return self.list_payment_schedules_with_http_info(limit, page_token, id, description, account_id, customer_id, **kwargs)  # noqa: E501

    @validate_arguments
    def list_payment_schedules_with_http_info(self, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, id : Annotated[Optional[conlist(StrictStr)], Field(description="IDs. Multiple IDs can be provided as a comma-separated list.")] = None, description : Annotated[Optional[StrictStr], Field(description="Payment schedule description. String can be provided with wildcard.")] = None, account_id : Annotated[Optional[conlist(StrictStr)], Field(description="Originating account IDs. Multiple IDs can be provided as a comma-separated list.")] = None, customer_id : Annotated[Optional[conlist(StrictStr)], Field(description="The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.")] = None, **kwargs):  # noqa: E501
        """List payment schedules  # noqa: E501

        Get paginated list of payment schedules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_payment_schedules_with_http_info(limit, page_token, id, description, account_id, customer_id, async_req=True)
        >>> result = thread.get()

        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param id: IDs. Multiple IDs can be provided as a comma-separated list.
        :type id: List[str]
        :param description: Payment schedule description. String can be provided with wildcard.
        :type description: str
        :param account_id: Originating account IDs. Multiple IDs can be provided as a comma-separated list.
        :type account_id: List[str]
        :param customer_id: The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.
        :type customer_id: List[str]
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaymentScheduleList, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'limit',
            'page_token',
            'id',
            'description',
            'account_id',
            'customer_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_payment_schedules" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('page_token') is not None:  # noqa: E501
            _query_params.append(('page_token', _params['page_token']))

        if _params.get('id') is not None:  # noqa: E501
            _query_params.append(('id', _params['id']))
            _collection_formats['id'] = 'csv'

        if _params.get('description') is not None:  # noqa: E501
            _query_params.append(('description', _params['description']))

        if _params.get('account_id') is not None:  # noqa: E501
            _query_params.append(('account_id', _params['account_id']))
            _collection_formats['account_id'] = 'csv'

        if _params.get('customer_id') is not None:  # noqa: E501
            _query_params.append(('customer_id', _params['customer_id']))
            _collection_formats['customer_id'] = 'csv'

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PaymentScheduleList",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/payment_schedules', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_payments(self, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, id : Annotated[Optional[conlist(StrictStr)], Field(description="IDs. Multiple IDs can be provided as a comma-separated list.")] = None, description : Annotated[Optional[StrictStr], Field(description="Payment schedule description. String can be provided with wildcard.")] = None, schedule_id : Annotated[Optional[conlist(StrictStr)], Field(description="Payment schedule IDs. Multiple IDs can be provided as a comma-separated list.")] = None, payment_type : Annotated[Optional[StrictStr], Field(description="Payment type")] = None, account_id : Annotated[Optional[conlist(StrictStr)], Field(description="Originating account IDs. Multiple IDs can be provided as a comma-separated list.")] = None, customer_id : Annotated[Optional[conlist(StrictStr)], Field(description="The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.")] = None, start_date : Annotated[Optional[date], Field(description="Payments that the execution date is later than the start date (inclusvie)")] = None, end_date : Annotated[Optional[date], Field(description="Payments that the execution date is earlier than the end date (exclusvie)")] = None, **kwargs) -> PaymentList:  # noqa: E501
        """List payments  # noqa: E501

        Get paginated list of payments  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_payments(limit, page_token, id, description, schedule_id, payment_type, account_id, customer_id, start_date, end_date, async_req=True)
        >>> result = thread.get()

        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param id: IDs. Multiple IDs can be provided as a comma-separated list.
        :type id: List[str]
        :param description: Payment schedule description. String can be provided with wildcard.
        :type description: str
        :param schedule_id: Payment schedule IDs. Multiple IDs can be provided as a comma-separated list.
        :type schedule_id: List[str]
        :param payment_type: Payment type
        :type payment_type: str
        :param account_id: Originating account IDs. Multiple IDs can be provided as a comma-separated list.
        :type account_id: List[str]
        :param customer_id: The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.
        :type customer_id: List[str]
        :param start_date: Payments that the execution date is later than the start date (inclusvie)
        :type start_date: date
        :param end_date: Payments that the execution date is earlier than the end date (exclusvie)
        :type end_date: date
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaymentList
        """
        kwargs['_return_http_data_only'] = True
        return self.list_payments_with_http_info(limit, page_token, id, description, schedule_id, payment_type, account_id, customer_id, start_date, end_date, **kwargs)  # noqa: E501

    @validate_arguments
    def list_payments_with_http_info(self, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, id : Annotated[Optional[conlist(StrictStr)], Field(description="IDs. Multiple IDs can be provided as a comma-separated list.")] = None, description : Annotated[Optional[StrictStr], Field(description="Payment schedule description. String can be provided with wildcard.")] = None, schedule_id : Annotated[Optional[conlist(StrictStr)], Field(description="Payment schedule IDs. Multiple IDs can be provided as a comma-separated list.")] = None, payment_type : Annotated[Optional[StrictStr], Field(description="Payment type")] = None, account_id : Annotated[Optional[conlist(StrictStr)], Field(description="Originating account IDs. Multiple IDs can be provided as a comma-separated list.")] = None, customer_id : Annotated[Optional[conlist(StrictStr)], Field(description="The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.")] = None, start_date : Annotated[Optional[date], Field(description="Payments that the execution date is later than the start date (inclusvie)")] = None, end_date : Annotated[Optional[date], Field(description="Payments that the execution date is earlier than the end date (exclusvie)")] = None, **kwargs):  # noqa: E501
        """List payments  # noqa: E501

        Get paginated list of payments  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_payments_with_http_info(limit, page_token, id, description, schedule_id, payment_type, account_id, customer_id, start_date, end_date, async_req=True)
        >>> result = thread.get()

        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param id: IDs. Multiple IDs can be provided as a comma-separated list.
        :type id: List[str]
        :param description: Payment schedule description. String can be provided with wildcard.
        :type description: str
        :param schedule_id: Payment schedule IDs. Multiple IDs can be provided as a comma-separated list.
        :type schedule_id: List[str]
        :param payment_type: Payment type
        :type payment_type: str
        :param account_id: Originating account IDs. Multiple IDs can be provided as a comma-separated list.
        :type account_id: List[str]
        :param customer_id: The IDs of customers who created the payment schedules. Multiple IDs can be provided as a comma-separated list.
        :type customer_id: List[str]
        :param start_date: Payments that the execution date is later than the start date (inclusvie)
        :type start_date: date
        :param end_date: Payments that the execution date is earlier than the end date (exclusvie)
        :type end_date: date
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaymentList, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'limit',
            'page_token',
            'id',
            'description',
            'schedule_id',
            'payment_type',
            'account_id',
            'customer_id',
            'start_date',
            'end_date'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_payments" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('page_token') is not None:  # noqa: E501
            _query_params.append(('page_token', _params['page_token']))

        if _params.get('id') is not None:  # noqa: E501
            _query_params.append(('id', _params['id']))
            _collection_formats['id'] = 'csv'

        if _params.get('description') is not None:  # noqa: E501
            _query_params.append(('description', _params['description']))

        if _params.get('schedule_id') is not None:  # noqa: E501
            _query_params.append(('schedule_id', _params['schedule_id']))
            _collection_formats['schedule_id'] = 'csv'

        if _params.get('payment_type') is not None:  # noqa: E501
            _query_params.append(('payment_type', _params['payment_type']))

        if _params.get('account_id') is not None:  # noqa: E501
            _query_params.append(('account_id', _params['account_id']))
            _collection_formats['account_id'] = 'csv'

        if _params.get('customer_id') is not None:  # noqa: E501
            _query_params.append(('customer_id', _params['customer_id']))
            _collection_formats['customer_id'] = 'csv'

        if _params.get('start_date') is not None:  # noqa: E501
            if isinstance(_params['start_date'], datetime):
                _query_parame.append(('start_date', _params['start_date'].strftime(self.api_client.configuration.date_format)))
            else:
                _query_params.append(('start_date', _params['start_date']))

        if _params.get('end_date') is not None:  # noqa: E501
            if isinstance(_params['end_date'], datetime):
                _query_parame.append(('end_date', _params['end_date'].strftime(self.api_client.configuration.date_format)))
            else:
                _query_params.append(('end_date', _params['end_date']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PaymentList",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/payment_schedules/payments', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def patch_payment_schedule(self, payment_schedule_id : Annotated[StrictStr, Field(..., description="Payment schedule ID")], patch_payment_schedule : Annotated[PatchPaymentSchedule, Field(..., description="payment schedule to update")], **kwargs) -> PaymentSchedule:  # noqa: E501
        """Update a payment schedule  # noqa: E501

        Update a payment schedule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.patch_payment_schedule(payment_schedule_id, patch_payment_schedule, async_req=True)
        >>> result = thread.get()

        :param payment_schedule_id: Payment schedule ID (required)
        :type payment_schedule_id: str
        :param patch_payment_schedule: payment schedule to update (required)
        :type patch_payment_schedule: PatchPaymentSchedule
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaymentSchedule
        """
        kwargs['_return_http_data_only'] = True
        return self.patch_payment_schedule_with_http_info(payment_schedule_id, patch_payment_schedule, **kwargs)  # noqa: E501

    @validate_arguments
    def patch_payment_schedule_with_http_info(self, payment_schedule_id : Annotated[StrictStr, Field(..., description="Payment schedule ID")], patch_payment_schedule : Annotated[PatchPaymentSchedule, Field(..., description="payment schedule to update")], **kwargs):  # noqa: E501
        """Update a payment schedule  # noqa: E501

        Update a payment schedule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.patch_payment_schedule_with_http_info(payment_schedule_id, patch_payment_schedule, async_req=True)
        >>> result = thread.get()

        :param payment_schedule_id: Payment schedule ID (required)
        :type payment_schedule_id: str
        :param patch_payment_schedule: payment schedule to update (required)
        :type patch_payment_schedule: PatchPaymentSchedule
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaymentSchedule, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'payment_schedule_id',
            'patch_payment_schedule'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method patch_payment_schedule" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['payment_schedule_id']:
            _path_params['payment_schedule_id'] = _params['payment_schedule_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['patch_payment_schedule']:
            _body_params = _params['patch_payment_schedule']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PaymentSchedule",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '409': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/payment_schedules/{payment_schedule_id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import date, datetime
from typing import Dict, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, constr, validator

class BaseCashPickup(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    account_id: StrictStr = ...
    amount: StrictInt = Field(..., description="The amount (in cents) of the transaction")
    business_id: Optional[StrictStr] = None
    creation_time: Optional[datetime] = None
    currency: Optional[constr(strict=True)] = None
    employee_customer_id: Optional[StrictStr] = None
    id: Optional[StrictStr] = None
    last_updated_time: Optional[datetime] = None
    metadata: Optional[Dict[str, StrictStr]] = None
    pickup_time: Optional[datetime] = Field(None, description="the date when the money was actually picked up")
    posted_amount: Optional[StrictInt] = None
    posted_date: Optional[date] = Field(None, description="The date the transaction was posted. This is the date any money is considered to be added or removed from an account.")
    reference_id: StrictStr = Field(..., description="An external ID provided by the partner. This is not guaranteed to be globally unique.")
    scheduled_pickup_date: Optional[date] = Field(None, description="the date when the money was expected to be picked up")
    transaction_id: Optional[StrictStr] = None
    __properties = ["account_id", "amount", "business_id", "creation_time", "currency", "employee_customer_id", "id", "last_updated_time", "metadata", "pickup_time", "posted_amount", "posted_date", "reference_id", "scheduled_pickup_date", "transaction_id"]

    @validator('currency')
    def currency_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"^[A-Z]{3}$", v):
            raise ValueError(r"must validate the regular expression /^[A-Z]{3}$/")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BaseCashPickup:
        """Create an instance of BaseCashPickup from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "creation_time",
                            "currency",
                            "id",
                            "last_updated_time",
                            "pickup_time",
                            "posted_amount",
                            "posted_date",
                            "transaction_id",
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BaseCashPickup:
        """Create an instance of BaseCashPickup from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return BaseCashPickup.parse_obj(obj)

        _obj = BaseCashPickup.parse_obj({
            "account_id": obj.get("account_id"),
            "amount": obj.get("amount"),
            "business_id": obj.get("business_id"),
            "creation_time": obj.get("creation_time"),
            "currency": obj.get("currency"),
            "employee_customer_id": obj.get("employee_customer_id"),
            "id": obj.get("id"),
            "last_updated_time": obj.get("last_updated_time"),
            "metadata": obj.get("metadata"),
            "pickup_time": obj.get("pickup_time"),
            "posted_amount": obj.get("posted_amount"),
            "posted_date": obj.get("posted_date"),
            "reference_id": obj.get("reference_id"),
            "scheduled_pickup_date": obj.get("scheduled_pickup_date"),
            "transaction_id": obj.get("transaction_id")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictStr, constr, validator
from synctera_client_ng.models.account_summary_balance_ceiling import AccountSummaryBalanceCeiling
from synctera_client_ng.models.account_summary_balance_floor import AccountSummaryBalanceFloor
from synctera_client_ng.models.financial_institution import FinancialInstitution

class AccountSummary(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    account_number: Optional[StrictStr] = Field(None, description="Account number")
    account_status: Optional[StrictStr] = Field(None, description="Account Status")
    account_type: Optional[StrictStr] = Field(None, description="The type of the account. In lead mode, this always takes the value of the template. If not specified in shadow mode, CHECKING will be assumed. ")
    balance_ceiling: Optional[AccountSummaryBalanceCeiling] = None
    balance_floor: Optional[AccountSummaryBalanceFloor] = None
    creation_time: Optional[datetime] = Field(None, description="Account creation time")
    currency: Optional[constr(strict=True)] = Field(None, description="Account currency or account settlement currency. ISO 4217 alphabetic currency code. Default USD")
    customer_type: Optional[StrictStr] = Field(None, description="Customer type")
    financial_institution: Optional[FinancialInstitution] = None
    id: Optional[StrictStr] = Field(None, description="The unique identifier of the account the statement belongs to")
    last_updated_time: Optional[datetime] = Field(None, description="Account last modification time")
    nickname: Optional[StrictStr] = Field(None, description="User provided account nickname")
    __properties = ["account_number", "account_status", "account_type", "balance_ceiling", "balance_floor", "creation_time", "currency", "customer_type", "financial_institution", "id", "last_updated_time", "nickname"]

    @validator('currency')
    def currency_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"^[A-Z]{3}$", v):
            raise ValueError(r"must validate the regular expression /^[A-Z]{3}$/")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AccountSummary:
        """Create an instance of AccountSummary from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of balance_ceiling
        if self.balance_ceiling:
            _dict['balance_ceiling'] = self.balance_ceiling.to_dict()
        # override the default output from pydantic by calling `to_dict()` of balance_floor
        if self.balance_floor:
            _dict['balance_floor'] = self.balance_floor.to_dict()
        # override the default output from pydantic by calling `to_dict()` of financial_institution
        if self.financial_institution:
            _dict['financial_institution'] = self.financial_institution.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AccountSummary:
        """Create an instance of AccountSummary from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AccountSummary.parse_obj(obj)

        _obj = AccountSummary.parse_obj({
            "account_number": obj.get("account_number"),
            "account_status": obj.get("account_status"),
            "account_type": obj.get("account_type"),
            "balance_ceiling": AccountSummaryBalanceCeiling.from_dict(obj.get("balance_ceiling")) if obj.get("balance_ceiling") is not None else None,
            "balance_floor": AccountSummaryBalanceFloor.from_dict(obj.get("balance_floor")) if obj.get("balance_floor") is not None else None,
            "creation_time": obj.get("creation_time"),
            "currency": obj.get("currency"),
            "customer_type": obj.get("customer_type"),
            "financial_institution": FinancialInstitution.from_dict(obj.get("financial_institution")) if obj.get("financial_institution") is not None else None,
            "id": obj.get("id"),
            "last_updated_time": obj.get("last_updated_time"),
            "nickname": obj.get("nickname")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import absolute_import

import re  # noqa: F401

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictStr, conint, conlist, constr, validator

from typing import Optional

from synctera_client_ng.models.ban_status import BanStatus
from synctera_client_ng.models.delete_response import DeleteResponse
from synctera_client_ng.models.patch_person import PatchPerson
from synctera_client_ng.models.patch_personal_id import PatchPersonalId
from synctera_client_ng.models.person_list import PersonList
from synctera_client_ng.models.post_person import PostPerson
from synctera_client_ng.models.post_personal_id_w_cust import PostPersonalIdWCust
from synctera_client_ng.models.prefill_request import PrefillRequest
from synctera_client_ng.models.response_person import ResponsePerson
from synctera_client_ng.models.response_personal_id_w_cust import ResponsePersonalIdWCust
from synctera_client_ng.models.status2 import Status2

from synctera_client_ng.api_client import ApiClient
from synctera_client_ng.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class PersonsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def create_person(self, post_person : PostPerson, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> ResponsePerson:  # noqa: E501
        """Create a person  # noqa: E501

        Create a person who may act as a personal customer or a director/officer/owner of a business. You can then verify the identity of this customer and associate them with other people and accounts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_person(post_person, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param post_person: (required)
        :type post_person: PostPerson
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ResponsePerson
        """
        kwargs['_return_http_data_only'] = True
        return self.create_person_with_http_info(post_person, idempotency_key, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def create_person_with_http_info(self, post_person : PostPerson, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Create a person  # noqa: E501

        Create a person who may act as a personal customer or a director/officer/owner of a business. You can then verify the identity of this customer and associate them with other people and accounts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_person_with_http_info(post_person, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param post_person: (required)
        :type post_person: PostPerson
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ResponsePerson, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'post_person',
            'idempotency_key',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_person" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['post_person']:
            _body_params = _params['post_person']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '201': "ResponsePerson",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '409': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def create_personal_id(self, post_personal_id_w_cust : PostPersonalIdWCust, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, **kwargs) -> ResponsePersonalIdWCust:  # noqa: E501
        """Create a personal identifier (beta)  # noqa: E501

        > 🚧 Beta > This is an Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint. Create a personal identifier, e.g. SSN, for this customer   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_personal_id(post_personal_id_w_cust, idempotency_key, async_req=True)
        >>> result = thread.get()

        :param post_personal_id_w_cust: (required)
        :type post_personal_id_w_cust: PostPersonalIdWCust
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ResponsePersonalIdWCust
        """
        kwargs['_return_http_data_only'] = True
        return self.create_personal_id_with_http_info(post_personal_id_w_cust, idempotency_key, **kwargs)  # noqa: E501

    @validate_arguments
    def create_personal_id_with_http_info(self, post_personal_id_w_cust : PostPersonalIdWCust, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, **kwargs):  # noqa: E501
        """Create a personal identifier (beta)  # noqa: E501

        > 🚧 Beta > This is an Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint. Create a personal identifier, e.g. SSN, for this customer   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_personal_id_with_http_info(post_personal_id_w_cust, idempotency_key, async_req=True)
        >>> result = thread.get()

        :param post_personal_id_w_cust: (required)
        :type post_personal_id_w_cust: PostPersonalIdWCust
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ResponsePersonalIdWCust, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'post_personal_id_w_cust',
            'idempotency_key'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_personal_id" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['post_personal_id_w_cust']:
            _body_params = _params['post_personal_id_w_cust']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '201': "ResponsePersonalIdWCust",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons/personal_ids', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def delete_personal_id(self, personal_id_id : Annotated[StrictStr, Field(..., description="UUID for a personal identifier.")], **kwargs) -> DeleteResponse:  # noqa: E501
        """Delete a personal identifier (beta)  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint. Delete personal identifier   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_personal_id(personal_id_id, async_req=True)
        >>> result = thread.get()

        :param personal_id_id: UUID for a personal identifier. (required)
        :type personal_id_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: DeleteResponse
        """
        kwargs['_return_http_data_only'] = True
        return self.delete_personal_id_with_http_info(personal_id_id, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_personal_id_with_http_info(self, personal_id_id : Annotated[StrictStr, Field(..., description="UUID for a personal identifier.")], **kwargs):  # noqa: E501
        """Delete a personal identifier (beta)  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint. Delete personal identifier   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_personal_id_with_http_info(personal_id_id, async_req=True)
        >>> result = thread.get()

        :param personal_id_id: UUID for a personal identifier. (required)
        :type personal_id_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(DeleteResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'personal_id_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_personal_id" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['personal_id_id']:
            _path_params['personal_id_id'] = _params['personal_id_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "DeleteResponse",
            '401': "Error",
            '404': "Error",
        }

        return self.api_client.call_api(
            '/persons/personal_ids/{personal_id_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_person(self, person_id : Annotated[StrictStr, Field(..., description="Person's unique identifier.")], partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> ResponsePerson:  # noqa: E501
        """Get person  # noqa: E501

        Get person by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_person(person_id, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param person_id: Person's unique identifier. (required)
        :type person_id: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ResponsePerson
        """
        kwargs['_return_http_data_only'] = True
        return self.get_person_with_http_info(person_id, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def get_person_with_http_info(self, person_id : Annotated[StrictStr, Field(..., description="Person's unique identifier.")], partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Get person  # noqa: E501

        Get person by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_person_with_http_info(person_id, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param person_id: Person's unique identifier. (required)
        :type person_id: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ResponsePerson, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'person_id',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_person" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['person_id']:
            _path_params['person_id'] = _params['person_id']


        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "ResponsePerson",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons/{person_id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_persons(self, id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the resource. Multiple IDs can be provided as a comma-separated list. ")] = None, first_name : Optional[StrictStr] = None, last_name : Optional[StrictStr] = None, phone_number : Optional[StrictStr] = None, email : Optional[StrictStr] = None, ssn_last_4 : Optional[constr(strict=True, max_length=4, min_length=4)] = None, ban_status : Annotated[Optional[conlist(BanStatus)], Field(description="(alpha) Ban status of the person. Multiple values can be provided as a comma-seperated list of the following: * `ALLOWED` – person is not banned or suspended * `SUSPENDED` - person is suspended * `BANNED` – person is banned ")] = None, status : Optional[Status2] = None, is_customer : Annotated[Optional[StrictBool], Field(description="If true, show only customers. If false, show non-customer parties.")] = None, tenant : Optional[StrictStr] = None, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, sort_by : Annotated[Optional[conlist(StrictStr)], Field(description="Specifies the sort order for the returned persons. ")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> PersonList:  # noqa: E501
        """List persons  # noqa: E501

        Retrieves paginated list of persons associated with the authorized requester.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_persons(id, first_name, last_name, phone_number, email, ssn_last_4, ban_status, status, is_customer, tenant, limit, page_token, sort_by, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param id: Unique identifier for the resource. Multiple IDs can be provided as a comma-separated list. 
        :type id: List[str]
        :param first_name:
        :type first_name: str
        :param last_name:
        :type last_name: str
        :param phone_number:
        :type phone_number: str
        :param email:
        :type email: str
        :param ssn_last_4:
        :type ssn_last_4: str
        :param ban_status: (alpha) Ban status of the person. Multiple values can be provided as a comma-seperated list of the following: * `ALLOWED` – person is not banned or suspended * `SUSPENDED` - person is suspended * `BANNED` – person is banned 
        :type ban_status: List[BanStatus]
        :param status:
        :type status: Status2
        :param is_customer: If true, show only customers. If false, show non-customer parties.
        :type is_customer: bool
        :param tenant:
        :type tenant: str
        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param sort_by: Specifies the sort order for the returned persons. 
        :type sort_by: List[str]
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PersonList
        """
        kwargs['_return_http_data_only'] = True
        return self.list_persons_with_http_info(id, first_name, last_name, phone_number, email, ssn_last_4, ban_status, status, is_customer, tenant, limit, page_token, sort_by, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def list_persons_with_http_info(self, id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the resource. Multiple IDs can be provided as a comma-separated list. ")] = None, first_name : Optional[StrictStr] = None, last_name : Optional[StrictStr] = None, phone_number : Optional[StrictStr] = None, email : Optional[StrictStr] = None, ssn_last_4 : Optional[constr(strict=True, max_length=4, min_length=4)] = None, ban_status : Annotated[Optional[conlist(BanStatus)], Field(description="(alpha) Ban status of the person. Multiple values can be provided as a comma-seperated list of the following: * `ALLOWED` – person is not banned or suspended * `SUSPENDED` - person is suspended * `BANNED` – person is banned ")] = None, status : Optional[Status2] = None, is_customer : Annotated[Optional[StrictBool], Field(description="If true, show only customers. If false, show non-customer parties.")] = None, tenant : Optional[StrictStr] = None, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, sort_by : Annotated[Optional[conlist(StrictStr)], Field(description="Specifies the sort order for the returned persons. ")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """List persons  # noqa: E501

        Retrieves paginated list of persons associated with the authorized requester.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_persons_with_http_info(id, first_name, last_name, phone_number, email, ssn_last_4, ban_status, status, is_customer, tenant, limit, page_token, sort_by, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param id: Unique identifier for the resource. Multiple IDs can be provided as a comma-separated list. 
        :type id: List[str]
        :param first_name:
        :type first_name: str
        :param last_name:
        :type last_name: str
        :param phone_number:
        :type phone_number: str
        :param email:
        :type email: str
        :param ssn_last_4:
        :type ssn_last_4: str
        :param ban_status: (alpha) Ban status of the person. Multiple values can be provided as a comma-seperated list of the following: * `ALLOWED` – person is not banned or suspended * `SUSPENDED` - person is suspended * `BANNED` – person is banned 
        :type ban_status: List[BanStatus]
        :param status:
        :type status: Status2
        :param is_customer: If true, show only customers. If false, show non-customer parties.
        :type is_customer: bool
        :param tenant:
        :type tenant: str
        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param sort_by: Specifies the sort order for the returned persons. 
        :type sort_by: List[str]
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PersonList, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id',
            'first_name',
            'last_name',
            'phone_number',
            'email',
            'ssn_last_4',
            'ban_status',
            'status',
            'is_customer',
            'tenant',
            'limit',
            'page_token',
            'sort_by',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_persons" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('id') is not None:  # noqa: E501
            _query_params.append(('id', _params['id']))
            _collection_formats['id'] = 'csv'

        if _params.get('first_name') is not None:  # noqa: E501
            _query_params.append(('first_name', _params['first_name']))

        if _params.get('last_name') is not None:  # noqa: E501
            _query_params.append(('last_name', _params['last_name']))

        if _params.get('phone_number') is not None:  # noqa: E501
            _query_params.append(('phone_number', _params['phone_number']))

        if _params.get('email') is not None:  # noqa: E501
            _query_params.append(('email', _params['email']))

        if _params.get('ssn_last_4') is not None:  # noqa: E501
            _query_params.append(('ssn_last_4', _params['ssn_last_4']))

        if _params.get('ban_status') is not None:  # noqa: E501
            _query_params.append(('ban_status', _params['ban_status']))
            _collection_formats['ban_status'] = 'csv'

        if _params.get('status') is not None:  # noqa: E501
            _query_params.append(('status', _params['status']))

        if _params.get('is_customer') is not None:  # noqa: E501
            _query_params.append(('is_customer', _params['is_customer']))

        if _params.get('tenant') is not None:  # noqa: E501
            _query_params.append(('tenant', _params['tenant']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('page_token') is not None:  # noqa: E501
            _query_params.append(('page_token', _params['page_token']))

        if _params.get('sort_by') is not None:  # noqa: E501
            _query_params.append(('sort_by', _params['sort_by']))
            _collection_formats['sort_by'] = 'csv'

        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PersonList",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def prefill_person(self, person_id : Annotated[StrictStr, Field(..., description="Person's unique identifier.")], prefill_request : PrefillRequest, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, **kwargs) -> PrefillRequest:  # noqa: E501
        """Prefill person  # noqa: E501

        Prefill takes a subset of a person's information and automatically fills personally identifiable information (PII) required for know your customer (KYC) verification.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.prefill_person(person_id, prefill_request, idempotency_key, async_req=True)
        >>> result = thread.get()

        :param person_id: Person's unique identifier. (required)
        :type person_id: str
        :param prefill_request: (required)
        :type prefill_request: PrefillRequest
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PrefillRequest
        """
        kwargs['_return_http_data_only'] = True
        return self.prefill_person_with_http_info(person_id, prefill_request, idempotency_key, **kwargs)  # noqa: E501

    @validate_arguments
    def prefill_person_with_http_info(self, person_id : Annotated[StrictStr, Field(..., description="Person's unique identifier.")], prefill_request : PrefillRequest, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, **kwargs):  # noqa: E501
        """Prefill person  # noqa: E501

        Prefill takes a subset of a person's information and automatically fills personally identifiable information (PII) required for know your customer (KYC) verification.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.prefill_person_with_http_info(person_id, prefill_request, idempotency_key, async_req=True)
        >>> result = thread.get()

        :param person_id: Person's unique identifier. (required)
        :type person_id: str
        :param prefill_request: (required)
        :type prefill_request: PrefillRequest
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PrefillRequest, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'person_id',
            'prefill_request',
            'idempotency_key'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method prefill_person" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['person_id']:
            _path_params['person_id'] = _params['person_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['prefill_request']:
            _body_params = _params['prefill_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PrefillRequest",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons/{person_id}/prefill', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def update_person(self, person_id : Annotated[StrictStr, Field(..., description="Person's unique identifier.")], patch_person : PatchPerson, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> ResponsePerson:  # noqa: E501
        """Update person  # noqa: E501

        Update person by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_person(person_id, patch_person, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param person_id: Person's unique identifier. (required)
        :type person_id: str
        :param patch_person: (required)
        :type patch_person: PatchPerson
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ResponsePerson
        """
        kwargs['_return_http_data_only'] = True
        return self.update_person_with_http_info(person_id, patch_person, idempotency_key, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def update_person_with_http_info(self, person_id : Annotated[StrictStr, Field(..., description="Person's unique identifier.")], patch_person : PatchPerson, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Update person  # noqa: E501

        Update person by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_person_with_http_info(person_id, patch_person, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param person_id: Person's unique identifier. (required)
        :type person_id: str
        :param patch_person: (required)
        :type patch_person: PatchPerson
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ResponsePerson, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'person_id',
            'patch_person',
            'idempotency_key',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_person" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['person_id']:
            _path_params['person_id'] = _params['person_id']


        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['patch_person']:
            _body_params = _params['patch_person']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "ResponsePerson",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons/{person_id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def update_personal_id(self, personal_id_id : Annotated[StrictStr, Field(..., description="UUID for a personal identifier.")], patch_personal_id : PatchPersonalId, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, **kwargs) -> ResponsePersonalIdWCust:  # noqa: E501
        """Update a personal identifier (beta)  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint. Update a personal identifier, e.g. SSN, for this customer   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_personal_id(personal_id_id, patch_personal_id, idempotency_key, async_req=True)
        >>> result = thread.get()

        :param personal_id_id: UUID for a personal identifier. (required)
        :type personal_id_id: str
        :param patch_personal_id: (required)
        :type patch_personal_id: PatchPersonalId
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ResponsePersonalIdWCust
        """
        kwargs['_return_http_data_only'] = True
        return self.update_personal_id_with_http_info(personal_id_id, patch_personal_id, idempotency_key, **kwargs)  # noqa: E501

    @validate_arguments
    def update_personal_id_with_http_info(self, personal_id_id : Annotated[StrictStr, Field(..., description="UUID for a personal identifier.")], patch_personal_id : PatchPersonalId, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, **kwargs):  # noqa: E501
        """Update a personal identifier (beta)  # noqa: E501

        > 🚧 Beta > This is a Beta endpoint. Feedback from the community is welcome. We may make breaking changes to this endpoint. Update a personal identifier, e.g. SSN, for this customer   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_personal_id_with_http_info(personal_id_id, patch_personal_id, idempotency_key, async_req=True)
        >>> result = thread.get()

        :param personal_id_id: UUID for a personal identifier. (required)
        :type personal_id_id: str
        :param patch_personal_id: (required)
        :type patch_personal_id: PatchPersonalId
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ResponsePersonalIdWCust, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'personal_id_id',
            'patch_personal_id',
            'idempotency_key'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_personal_id" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['personal_id_id']:
            _path_params['personal_id_id'] = _params['personal_id_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['patch_personal_id']:
            _body_params = _params['patch_personal_id']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '201': "ResponsePersonalIdWCust",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/persons/personal_ids/{personal_id_id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from inspect import getfullargspec
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class VerificationType1(str, Enum):
    """
    The verification run on the customer. One the following:   * `IDENTITY` – verify that the information provided is associated with the identity of a real person or business.   * `WATCHLIST` – checks watchlists for known fraud, money laundering, and other suspicious activity.   * `RELATED_ENTITIES` – represents dependent verification checks for related parties (e.g. all beneficial owners must pass KYC for a business to pass KYB).   * `MANUAL_REVIEW` – represents the outcome of a manual review of the verification done on a party (note: overrides the outcome of other verification types). 
    """

    """
    allowed enum values
    """
    IDENTITY = 'IDENTITY'
    WATCHLIST = 'WATCHLIST'
    RELATED_ENTITIES = 'RELATED_ENTITIES'
    MANUAL_REVIEW = 'MANUAL_REVIEW'




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import absolute_import

import re  # noqa: F401

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, StrictStr, conint, conlist, validator

from typing import Optional

from synctera_client_ng.models.delete_response import DeleteResponse
from synctera_client_ng.models.patch_relationship_in import PatchRelationshipIn
from synctera_client_ng.models.relationship_in import RelationshipIn
from synctera_client_ng.models.relationships_list import RelationshipsList

from synctera_client_ng.api_client import ApiClient
from synctera_client_ng.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class RelationshipsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def create_relationship(self, relationship_in : Annotated[RelationshipIn, Field(..., description="Party relationship to create.")], idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> RelationshipIn:  # noqa: E501
        """Create a relationship  # noqa: E501

        Create a party relationship.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_relationship(relationship_in, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_in: Party relationship to create. (required)
        :type relationship_in: RelationshipIn
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RelationshipIn
        """
        kwargs['_return_http_data_only'] = True
        return self.create_relationship_with_http_info(relationship_in, idempotency_key, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def create_relationship_with_http_info(self, relationship_in : Annotated[RelationshipIn, Field(..., description="Party relationship to create.")], idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Create a relationship  # noqa: E501

        Create a party relationship.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_relationship_with_http_info(relationship_in, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_in: Party relationship to create. (required)
        :type relationship_in: RelationshipIn
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RelationshipIn, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'relationship_in',
            'idempotency_key',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_relationship" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['relationship_in']:
            _body_params = _params['relationship_in']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '201': "RelationshipIn",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '409': "Error",
            '422': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/relationships', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def delete_relationship(self, relationship_id : Annotated[StrictStr, Field(..., description="Relationship ID.")], partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> DeleteResponse:  # noqa: E501
        """Delete relationship  # noqa: E501

        Delete party relationship by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_relationship(relationship_id, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_id: Relationship ID. (required)
        :type relationship_id: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: DeleteResponse
        """
        kwargs['_return_http_data_only'] = True
        return self.delete_relationship_with_http_info(relationship_id, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_relationship_with_http_info(self, relationship_id : Annotated[StrictStr, Field(..., description="Relationship ID.")], partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Delete relationship  # noqa: E501

        Delete party relationship by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_relationship_with_http_info(relationship_id, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_id: Relationship ID. (required)
        :type relationship_id: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(DeleteResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'relationship_id',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_relationship" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['relationship_id']:
            _path_params['relationship_id'] = _params['relationship_id']


        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "DeleteResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/relationships/{relationship_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_relationship(self, relationship_id : Annotated[StrictStr, Field(..., description="Relationship ID.")], partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> RelationshipIn:  # noqa: E501
        """Get relationship  # noqa: E501

        Get relationship by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_relationship(relationship_id, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_id: Relationship ID. (required)
        :type relationship_id: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RelationshipIn
        """
        kwargs['_return_http_data_only'] = True
        return self.get_relationship_with_http_info(relationship_id, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def get_relationship_with_http_info(self, relationship_id : Annotated[StrictStr, Field(..., description="Relationship ID.")], partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Get relationship  # noqa: E501

        Get relationship by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_relationship_with_http_info(relationship_id, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_id: Relationship ID. (required)
        :type relationship_id: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RelationshipIn, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'relationship_id',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_relationship" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['relationship_id']:
            _path_params['relationship_id'] = _params['relationship_id']


        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "RelationshipIn",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/relationships/{relationship_id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_relationships(self, id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique resource identifier")] = None, from_person_id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the person. Multiple IDs can be provided as a comma-separated list. ")] = None, from_business_id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. ")] = None, to_business_id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. ")] = None, relationship_type : Annotated[Optional[conlist(StrictStr)], Field(description="The relationship type, any of the following.  All direct businesses customers need at least one beneficial owner or controlling person. * `BENEFICIAL_OWNER_OF` – a person who directly or indirectly owns a portion of the business. * `MANAGING_PERSON_OF` – a person who is an officer, director, or controlling person of an organization. * `OWNER_OF` – a business with ownership of another business. ")] = None, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> RelationshipsList:  # noqa: E501
        """List relationships  # noqa: E501

        Retrieves paginated list of relationships viewable by the authorized requester.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_relationships(id, from_person_id, from_business_id, to_business_id, relationship_type, limit, page_token, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param id: Unique resource identifier
        :type id: List[str]
        :param from_person_id: Unique identifier for the person. Multiple IDs can be provided as a comma-separated list. 
        :type from_person_id: List[str]
        :param from_business_id: Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. 
        :type from_business_id: List[str]
        :param to_business_id: Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. 
        :type to_business_id: List[str]
        :param relationship_type: The relationship type, any of the following.  All direct businesses customers need at least one beneficial owner or controlling person. * `BENEFICIAL_OWNER_OF` – a person who directly or indirectly owns a portion of the business. * `MANAGING_PERSON_OF` – a person who is an officer, director, or controlling person of an organization. * `OWNER_OF` – a business with ownership of another business. 
        :type relationship_type: List[str]
        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RelationshipsList
        """
        kwargs['_return_http_data_only'] = True
        return self.list_relationships_with_http_info(id, from_person_id, from_business_id, to_business_id, relationship_type, limit, page_token, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def list_relationships_with_http_info(self, id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique resource identifier")] = None, from_person_id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the person. Multiple IDs can be provided as a comma-separated list. ")] = None, from_business_id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. ")] = None, to_business_id : Annotated[Optional[conlist(StrictStr)], Field(description="Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. ")] = None, relationship_type : Annotated[Optional[conlist(StrictStr)], Field(description="The relationship type, any of the following.  All direct businesses customers need at least one beneficial owner or controlling person. * `BENEFICIAL_OWNER_OF` – a person who directly or indirectly owns a portion of the business. * `MANAGING_PERSON_OF` – a person who is an officer, director, or controlling person of an organization. * `OWNER_OF` – a business with ownership of another business. ")] = None, limit : Optional[conint(strict=True, ge=1)] = None, page_token : Optional[StrictStr] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """List relationships  # noqa: E501

        Retrieves paginated list of relationships viewable by the authorized requester.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_relationships_with_http_info(id, from_person_id, from_business_id, to_business_id, relationship_type, limit, page_token, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param id: Unique resource identifier
        :type id: List[str]
        :param from_person_id: Unique identifier for the person. Multiple IDs can be provided as a comma-separated list. 
        :type from_person_id: List[str]
        :param from_business_id: Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. 
        :type from_business_id: List[str]
        :param to_business_id: Unique identifier for the business. Multiple IDs can be provided as a comma-separated list. 
        :type to_business_id: List[str]
        :param relationship_type: The relationship type, any of the following.  All direct businesses customers need at least one beneficial owner or controlling person. * `BENEFICIAL_OWNER_OF` – a person who directly or indirectly owns a portion of the business. * `MANAGING_PERSON_OF` – a person who is an officer, director, or controlling person of an organization. * `OWNER_OF` – a business with ownership of another business. 
        :type relationship_type: List[str]
        :param limit:
        :type limit: int
        :param page_token:
        :type page_token: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RelationshipsList, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id',
            'from_person_id',
            'from_business_id',
            'to_business_id',
            'relationship_type',
            'limit',
            'page_token',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_relationships" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('id') is not None:  # noqa: E501
            _query_params.append(('id', _params['id']))
            _collection_formats['id'] = 'csv'

        if _params.get('from_person_id') is not None:  # noqa: E501
            _query_params.append(('from_person_id', _params['from_person_id']))
            _collection_formats['from_person_id'] = 'csv'

        if _params.get('from_business_id') is not None:  # noqa: E501
            _query_params.append(('from_business_id', _params['from_business_id']))
            _collection_formats['from_business_id'] = 'csv'

        if _params.get('to_business_id') is not None:  # noqa: E501
            _query_params.append(('to_business_id', _params['to_business_id']))
            _collection_formats['to_business_id'] = 'csv'

        if _params.get('relationship_type') is not None:  # noqa: E501
            _query_params.append(('relationship_type', _params['relationship_type']))
            _collection_formats['relationship_type'] = 'csv'

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('page_token') is not None:  # noqa: E501
            _query_params.append(('page_token', _params['page_token']))

        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "RelationshipsList",
            '401': "Error",
            '403': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/relationships', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def update_relationship(self, relationship_id : Annotated[StrictStr, Field(..., description="Relationship ID.")], patch_relationship_in : PatchRelationshipIn, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs) -> PatchRelationshipIn:  # noqa: E501
        """Update relationship  # noqa: E501

        Update relationship by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_relationship(relationship_id, patch_relationship_in, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_id: Relationship ID. (required)
        :type relationship_id: str
        :param patch_relationship_in: (required)
        :type patch_relationship_in: PatchRelationshipIn
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PatchRelationshipIn
        """
        kwargs['_return_http_data_only'] = True
        return self.update_relationship_with_http_info(relationship_id, patch_relationship_in, idempotency_key, partner_ownership, **kwargs)  # noqa: E501

    @validate_arguments
    def update_relationship_with_http_info(self, relationship_id : Annotated[StrictStr, Field(..., description="Relationship ID.")], patch_relationship_in : PatchRelationshipIn, idempotency_key : Annotated[Optional[StrictStr], Field(description="An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.")] = None, partner_ownership : Annotated[Optional[StrictStr], Field(description="Return ownership resources for the specified partner. ")] = None, **kwargs):  # noqa: E501
        """Update relationship  # noqa: E501

        Update relationship by ID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_relationship_with_http_info(relationship_id, patch_relationship_in, idempotency_key, partner_ownership, async_req=True)
        >>> result = thread.get()

        :param relationship_id: Relationship ID. (required)
        :type relationship_id: str
        :param patch_relationship_in: (required)
        :type patch_relationship_in: PatchRelationshipIn
        :param idempotency_key: An idempotency key is an arbitrary unique value generated by client to detect subsequent retries of the same request. It is recommended that a UUID or a similar random identifier be used as an idempotency key. A different key must be used for each request, unless it is a retry.
        :type idempotency_key: str
        :param partner_ownership: Return ownership resources for the specified partner. 
        :type partner_ownership: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PatchRelationshipIn, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'relationship_id',
            'patch_relationship_in',
            'idempotency_key',
            'partner_ownership'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_relationship" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['relationship_id']:
            _path_params['relationship_id'] = _params['relationship_id']


        # process the query parameters
        _query_params = []
        if _params.get('partner_ownership') is not None:  # noqa: E501
            _query_params.append(('partner_ownership', _params['partner_ownership']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        if _params['idempotency_key']:
            _header_params['Idempotency-Key'] = _params['idempotency_key']

        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['patch_relationship_in']:
            _body_params = _params['patch_relationship_in']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json', 'application/problem+json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PatchRelationshipIn",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '409': "Error",
            '422': "Error",
            '500': "Error",
        }

        return self.api_client.call_api(
            '/relationships/{relationship_id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, constr, validator
from synctera_client_ng.models.merchant import Merchant
from synctera_client_ng.models.transfer_type import TransferType

class TransferResponseBase(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    account_id: StrictStr = Field(..., description="The ID of the account to which the card will be linked")
    amount: StrictInt = Field(..., description="Amount of the transfer in cents")
    country_code: StrictStr = Field(..., description="ISO-3166-1 Alpha-2 country code")
    created_time: datetime = ...
    currency: constr(strict=True, max_length=3, min_length=3) = Field(..., description="ISO 4217  Alpha-3 currency code")
    customer_id: StrictStr = Field(..., description="The ID of the customer to whom the card belongs")
    external_card_id: StrictStr = Field(..., description="The ID of the external card from/to which the transfer was initiated/received")
    id: StrictStr = Field(..., description="The ID of the transfer")
    last_modified_time: datetime = ...
    merchant: Merchant = ...
    status: StrictStr = Field(..., description="The status of the transfer")
    transaction_id: Optional[StrictStr] = Field(None, description="The transaction ID")
    type: TransferType = ...
    __properties = ["account_id", "amount", "country_code", "created_time", "currency", "customer_id", "external_card_id", "id", "last_modified_time", "merchant", "status", "transaction_id", "type"]

    @validator('status')
    def status_validate_enum(cls, v):
        if v not in ('SUCCEEDED', 'PENDING', 'DECLINED', 'CANCELED', 'UNKNOWN'):
            raise ValueError("must validate the enum values ('SUCCEEDED', 'PENDING', 'DECLINED', 'CANCELED', 'UNKNOWN')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransferResponseBase:
        """Create an instance of TransferResponseBase from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of merchant
        if self.merchant:
            _dict['merchant'] = self.merchant.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransferResponseBase:
        """Create an instance of TransferResponseBase from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TransferResponseBase.parse_obj(obj)

        _obj = TransferResponseBase.parse_obj({
            "account_id": obj.get("account_id"),
            "amount": obj.get("amount"),
            "country_code": obj.get("country_code"),
            "created_time": obj.get("created_time"),
            "currency": obj.get("currency"),
            "customer_id": obj.get("customer_id"),
            "external_card_id": obj.get("external_card_id"),
            "id": obj.get("id"),
            "last_modified_time": obj.get("last_modified_time"),
            "merchant": Merchant.from_dict(obj.get("merchant")) if obj.get("merchant") is not None else None,
            "status": obj.get("status"),
            "transaction_id": obj.get("transaction_id"),
            "type": obj.get("type")
        })
        return _obj




# coding: utf-8

"""
    Synctera API

    <h2>Let's build something great.</h2><p>Welcome to the official reference documentation for Synctera APIs. Our APIs are the best way to automate your company's banking needs and are designed to be easy to understand and implement.</p><p>We're continuously growing this library and what you see here is just the start, but if you need something specific or have a question, <a class='text-blue-600' href='https://synctera.com/contact' target='_blank' rel='noreferrer'>contact us</a>.</p>  # noqa: E501

    The version of the OpenAPI document: 0.41.0
    Contact: dev@swadesh.co
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
from inspect import getfullargspec
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictStr, constr, validator
from synctera_client_ng.models.address import Address
from synctera_client_ng.models.ban_status import BanStatus

class PatchCustomer(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    ban_status: Optional[BanStatus] = None
    dob: Optional[date] = Field(None, description="Customer's date of birth in RFC 3339 full-date format (YYYY-MM-DD)")
    email: Optional[StrictStr] = Field(None, description="Customer's email")
    first_name: Optional[StrictStr] = Field(None, description="Customer's first name")
    last_name: Optional[StrictStr] = Field(None, description="Customer's last name")
    legal_address: Optional[Address] = None
    metadata: Optional[Dict[str, Any]] = Field(None, description="User-supplied JSON format metadata. Do not use to store PII.")
    middle_name: Optional[StrictStr] = Field(None, description="Customer's middle name")
    note: Optional[StrictStr] = Field(None, description="A note is required when updating a customer's ban_status between SUSPENDED and ALLOWED")
    phone_number: Optional[constr(strict=True)] = Field(None, description="Customer's mobile phone number with country code in E.164 format. Must have a valid country code. Area code and local phone number are not validated.")
    shipping_address: Optional[Address] = None
    ssn: Optional[StrictStr] = Field(None, description="Customer's full tax ID eg SSN formatted with hyphens. This optional parameter is required when running KYC on a customer. Must be compiled with ^\\d{3}-\\d{2}-\\d{4}$. Response contains the last 4 digits only (e.g. 6789).")
    status: Optional[StrictStr] = Field(None, description="Customer's status")
    __properties = ["ban_status", "dob", "email", "first_name", "last_name", "legal_address", "metadata", "middle_name", "note", "phone_number", "shipping_address", "ssn", "status"]

    @validator('phone_number')
    def phone_number_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"^\+[1-9]\d{1,14}$", v):
            raise ValueError(r"must validate the regular expression /^\+[1-9]\d{1,14}$/")
        return v

    @validator('status')
    def status_validate_enum(cls, v):
        if v is None:
            return v

        if v not in ('ACTIVE', 'ESCHEAT', 'DECEASED', 'DENIED', 'DORMANT', 'FROZEN', 'INACTIVE', 'PROSPECT', 'SANCTION'):
            raise ValueError("must validate the enum values ('ACTIVE', 'ESCHEAT', 'DECEASED', 'DENIED', 'DORMANT', 'FROZEN', 'INACTIVE', 'PROSPECT', 'SANCTION')")
        return v

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PatchCustomer:
        """Create an instance of PatchCustomer from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of legal_address
        if self.legal_address:
            _dict['legal_address'] = self.legal_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of shipping_address
        if self.shipping_address:
            _dict['shipping_address'] = self.shipping_address.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PatchCustomer:
        """Create an instance of PatchCustomer from a dict"""
        if obj is None:
            return None

        if type(obj) is not dict:
            return PatchCustomer.parse_obj(obj)

        _obj = PatchCustomer.parse_obj({
            "ban_status": obj.get("ban_status"),
            "dob": obj.get("dob"),
            "email": obj.get("email"),
            "first_name": obj.get("first_name"),
            "last_name": obj.get("last_name"),
            "legal_address": Address.from_dict(obj.get("legal_address")) if obj.get("legal_address") is not None else None,
            "metadata": obj.get("metadata"),
            "middle_name": obj.get("middle_name"),
            "note": obj.get("note"),
            "phone_number": obj.get("phone_number"),
            "shipping_address": Address.from_dict(obj.get("shipping_address")) if obj.get("shipping_address") is not None else None,
            "ssn": obj.get("ssn"),
            "status": obj.get("status")
        })
        return _obj

